package com.dzeio.openhealth.services

import android.app.Notification
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.content.Intent
import android.os.Build
import android.os.IBinder
import android.util.Log
import android.widget.Toast
import androidx.core.app.NotificationCompat
import androidx.navigation.NavDeepLinkBuilder
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.R
import com.dzeio.openhealth.data.AppDatabase
import com.dzeio.openhealth.data.step.Step
import com.dzeio.openhealth.data.step.StepRepository
import com.dzeio.openhealth.data.step.StepRepository_Factory
import com.dzeio.openhealth.data.step.StepSource
import com.dzeio.openhealth.interfaces.NotificationChannels
import com.dzeio.openhealth.interfaces.NotificationIds
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.withTimeoutOrNull

class OpenHealthService : Service() {

    companion object {
        private const val TAG = "${Application.TAG}/Service"
    }

    private val stepRepository: StepRepository
        get() = StepRepository_Factory.newInstance(
            AppDatabase.getInstance(applicationContext).stepDao()
        )

    private var mNM: NotificationManager? = null
    private lateinit var notification: Notification

    /**
     * Displayed steps in the notification panel
     */
    private var stepsTaken = 0u

    /**
     * Need a buffer because sometime it does not complete the function
     */
    private var stepsBuffer = 0

    /**
     * Coroutine shit
     */
    private val job = SupervisorJob()
    private val scope = CoroutineScope(Dispatchers.IO + job)

    override fun onCreate() {
        mNM = getSystemService(NOTIFICATION_SERVICE) as NotificationManager?
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        Log.i("LocalService", "Received start id $startId: $intent")

        scope.launch {
            val source = StepSource(this@OpenHealthService)
            source.events.receiveAsFlow().collectLatest {
                Log.d(TAG, "Received value: $it")

                if (it <= 0f) {
                    Log.d(TAG, "No new steps registered ($it)")
                    return@collectLatest
                }
                Log.d(TAG, "New steps registered: $it")
                stepsTaken += it.toUInt()
                stepsBuffer += it.toInt()
                showNotification()
                val step = withTimeoutOrNull(1000) {
                    return@withTimeoutOrNull stepRepository.currentStep().firstOrNull()
                }
                Log.d(TAG, "stepRepository: $step")
                if (step != null) {
                    step.value += stepsBuffer
                    stepRepository.updateStep(step)
                } else {
                    stepRepository.addStep(Step(value = stepsBuffer))
                }
                stepsBuffer = 0
                Log.d(TAG, "Added step!")
            }
        }

        // Display a notification about us starting.  We put an icon in the status bar.

        startForeground(NotificationIds.Service.ordinal, showNotification())

        return START_STICKY
    }

    override fun onDestroy() {
        stopForeground(true)

        // Tell the user we stopped.
        Toast.makeText(this, "Service stopped", Toast.LENGTH_SHORT).show()

        super.onDestroy()
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }

    /**
     * Show a notification while this service is running.
     */
    private fun showNotification(): Notification {
        // The PendingIntent to launch our activity if the user selects this notification
        val flag =
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) PendingIntent.FLAG_IMMUTABLE else 0
        val intent = NavDeepLinkBuilder(this)
            .setGraph(R.navigation.mobile_navigation)
            .setDestination(R.id.nav_home)
            // Will nav to water home when there will be a way to add it there
//             .setDestination(R.id.nav_water_home)
            .createTaskStackBuilder()
            .getPendingIntent(0, flag)

        // Set the info for the views that show in the notification panel.
        this.notification = NotificationCompat.Builder(
            this,
            NotificationChannels.SERVICE.id
        )
            .setSmallIcon(R.drawable.ic_logo_small)
//            .setTicker("Pouet") // the status text
            .setWhen(System.currentTimeMillis()) // the time stamp
            .setContentTitle("Open Health Service") // the label of the entry
            .setContentText("Watching for your steps ($stepsTaken)") // the contents of the entry
            .setContentIntent(intent) // The intent to send when the entry is clicked
            .build()

        // Send the notification.
        mNM!!.notify(NotificationIds.Service.ordinal, notification)

        return notification
    }
}
