package com.dzeio.openhealth.ui.home

import android.content.SharedPreferences
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Application.Companion.TAG
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.water.WaterRepository
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.UnitFactory
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@HiltViewModel
class HomeViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    private val waterRepository: WaterRepository,
    settings: SharedPreferences
) : BaseViewModel() {

    private val _water = MutableLiveData<Water?>(null)
    val water: LiveData<Water?> = _water

    var waterCupSize = settings.getInt("water_cup_size", 200)

    var waterUnit =
        UnitFactory.volume(settings.getString("water_unit", "milliliter") ?: "Milliliter")

    var weightUnit =
        UnitFactory.mass(settings.getString("weight_unit", "kilogram") ?: "kilogram")

    val goalWeight: Int? =
        (settings.getString("weight_goal", null)?.toIntOrNull())

    val dailyWaterIntake: Int =
        ((settings.getString("water_intake", "1200")?.toFloatOrNull() ?: 1200f) * waterUnit.modifier)
            .toInt()

    init {
        viewModelScope.launch {
            waterRepository.todayWater().collectLatest {
                _water.value = it
            }
        }
        // don't listen for prefs changes
        settings.registerOnSharedPreferenceChangeListener { _, key ->
            Log.d(TAG, "Pref changed: $key")
            when (key) {
                "water_cup_size" -> {
                    waterCupSize = settings.getInt("water_cup_size", 200)
                }
            }
        }
    }

    /**
     * @deprecated
     */
    fun fetchWeights() = weightRepository.getWeights()

    /**
     * @deprecated
     */
    fun lastWeight() = weightRepository.lastWeight()

    fun fetchWeight(id: Long) = weightRepository.getWeight(id)

    suspend fun deleteWeight(weight: Weight) = weightRepository.deleteWeight(weight)

    suspend fun addWeight(weight: Weight) = weightRepository.addWeight(weight)

    fun fetchTodayWater() = waterRepository.todayWater()

    fun updateWater(water: Water) {
        viewModelScope.launch {
            waterRepository.addWater(water)
        }
    }

    fun deleteWater(item: Water) {
        viewModelScope.launch {
            waterRepository.deleteWater(item)
            _water.postValue(null)
        }
    }
}
